/* 
 * Adium is the legal property of its developers, whose names are listed in the copyright file included
 * with this source distribution.
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; if not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#import "AIContactOnlineSincePlugin.h"
#import "AIInterfaceController.h"
#import <Adium/AIListObject.h>
#import <Adium/AIListContact.h>
#import <AIUtilities/ESDateFormatterAdditions.h>

@implementation AIContactOnlineSincePlugin

- (void)installPlugin
{
    //Install our tooltip entry
    [[adium interfaceController] registerContactListTooltipEntry:self secondaryEntry:NO];
}

/*!
 * @brief Tooltip label
 *
 * @result A label, or nil if no tooltip entry should be shown
 */
- (NSString *)labelForObject:(AIListObject *)inObject
{
    return(AILocalizedString(@"Online Since", "This tooltip identifier will followd by a date"));
}

/*!
 * @brief Tooltip entry
 *
 * @result The tooltip entry, or nil if no tooltip should be shown
 */
- (NSAttributedString *)entryForObject:(AIListObject *)inObject
{
    NSAttributedString * entry = nil;
    if([inObject integerStatusObjectForKey:@"Online"]){

        NSDate	*signonDate;
	
        if(signonDate = [inObject statusObjectForKey:@"Signon Date"]){
            NSString		*currentDay, *signonDay, *signonTime;
            NSDateFormatter	*dayFormatter, *timeFormatter;
            
            //Create the formatters
            dayFormatter = [NSDateFormatter localizedShortDateFormatter];
            timeFormatter = [[NSDateFormatter alloc] initWithDateFormat:[NSDateFormatter localizedDateFormatStringShowingSeconds:NO 
																												   showingAMorPM:YES] 
												   allowNaturalLanguage:YES];
            
            //Get day & time strings
            currentDay = [dayFormatter stringForObjectValue:[NSDate date]];
            signonDay = [dayFormatter stringForObjectValue:signonDate];
            signonTime = [timeFormatter stringForObjectValue:signonDate];
            
            if([currentDay isEqualToString:signonDay]){ //Show time
                entry = [[NSAttributedString alloc] initWithString:signonTime];
                
            }else{ //Show date and time
                entry = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@, %@", signonDay, signonTime]];

            }
			
			[timeFormatter release];
        }
    }

    return([entry autorelease]);
}

@end
